@@ -43,8 +43,11 @@ class FalConnectionManager(PythonConnectionManager):
 @dataclass
 class FalCredentials(Credentials):
     default_environment: str = "local"
-
     teleport: Optional[TeleportCredentials] = None
+    host: str = ''
+    key_secret: str = ''
+    key_id: str = ''
+
 
     # NOTE: So we are allowed to not set them in profiles.yml
     # they are ignored for now
